@@ -7,7 +7,7 @@ use warnings;
 use English qw< -no_match_vars >;
 use Data::Dumper;
 use Scalar::Util qw< blessed >;
-our $VERSION = '0.726';
+our $VERSION = '0.728';
 
 use Log::Log4perl::Tiny
   qw< :easy :dead_if_first get_logger LOGLEVEL LEVELID_FOR >;
@@ -146,7 +146,8 @@ sub pipeline {
 }
 
 sub sequence {
-   my %args = normalize_args(@_, {name => 'sequence'});
+   my %args =
+     normalize_args(@_, [{name => 'sequence', tubes => []}, 'tubes']);
    identify(\%args);
 
    # cope with an empty list of tubes - equivalent to an "id" function but
